package me.goosemonkey.NoSpawnEggsV2;

import org.bukkit.ChatColor;
import org.bukkit.GameMode;
import org.bukkit.World;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;

public class ListenerPlayerSpawnEgg implements Listener
{
	static final String globalName = "$global";
	
	@EventHandler
	public void onPlayerInteract(PlayerInteractEvent event)
	{
		if (event.getPlayer().getItemInHand().getTypeId() != 383)
			return;
		
		if (event.getAction() != Action.RIGHT_CLICK_AIR && event.getAction() != Action.RIGHT_CLICK_BLOCK)
			return;
		
		if (!NoSpawnEggs.getMainConfig().getBoolean("blocking.spawnEggs.enable", true))
			return;
		
		if (NoSpawnEggs.getMainConfig().getBoolean("blocking.spawnEggs.onlyBlockCreative", false) && event.getPlayer().getGameMode() == GameMode.SURVIVAL)
			return;
		
		MobType type = ListenerPlayerSpawnEgg.getMobTypeFromId(event.getPlayer().getItemInHand().getDurability());
		
		if (NoSpawnEggs.hasPermission(event.getPlayer(), "nse.egg." + type.getCategory().name().toLowerCase() + "." + type.getPermName()))
			return;
		
		if (ListenerPlayerSpawnEgg.shouldSpawnBeBlocked(type, event.getPlayer().getWorld()))
		{
			event.setCancelled(true);
			
			event.getPlayer().sendMessage(ChatColor.GOLD + String.format(NoSpawnEggs.getLocaleConfig().getString("message.noSpawnerEggPerms", "You don't have permission to spawn this %s."), ChatColor.GREEN + type.getProperName() + ChatColor.GOLD));
		}
	}
	
	/**
	 * Assumes that the player has no permissions
	 */
	static boolean shouldSpawnBeBlocked(MobType type, World world)
	{
		FileConfiguration config = NoSpawnEggs.getMainConfig();
		
		if (config.getIntegerList("blocking.spawnEggs." + world.getName() + ".blockedIds") != null)
			if (config.getIntegerList("blocking.spawnEggs." + world.getName() + ".blockedIds").contains(type.getEntityId()))
				return true;
		
		if (config.getIntegerList("blocking.spawnEggs." + world.getName() + ".allowedIds") != null)
			if (config.getIntegerList("blocking.spawnEggs." + world.getName() + ".allowedIds").contains(type.getEntityId()))
				return false;
		
		if (config.isBoolean("blocking.spawnEggs." + world.getName() + "." + type.getCategory().name().toLowerCase()))
			return config.getBoolean("blocking.spawnEggs." + world.getName() + "." + type.getCategory().name().toLowerCase());

		
		if (config.getIntegerList("blocking.spawnEggs." + globalName + ".blockedIds") != null)
			if (config.getIntegerList("blocking.spawnEggs." + globalName + ".blockedIds").contains(type.getEntityId()))
				return true;
		
		if (config.getIntegerList("blocking.spawnEggs." + globalName + ".allowedIds") != null)
			if (config.getIntegerList("blocking.spawnEggs." + globalName + ".allowedIds").contains(type.getEntityId()))
				return false;
		
		if (config.isBoolean("blocking.spawnEggs." + globalName + "." + type.getCategory().name().toLowerCase()))
			return config.getBoolean("blocking.spawnEggs." + globalName + "." + type.getCategory().name().toLowerCase());
		
		return false;
	}
	
	enum MobType
	{
		CREEPER(50, MobCategory.MONSTER),
		SKELETON(51, MobCategory.MONSTER),
		SPIDER(52, MobCategory.MONSTER),
		ZOMBIE(54, MobCategory.MONSTER),
		SLIME(55, MobCategory.MONSTER),
		GHAST(56, MobCategory.MONSTER),
		PIGZOMBIE(57, MobCategory.MONSTER),
		ENDERMAN(58, MobCategory.MONSTER),
		CAVESPIDER(59, MobCategory.MONSTER),
		SILVERFISH(60, MobCategory.MONSTER),
		BLAZE(61, MobCategory.MONSTER),
		MAGMACUBE(62, MobCategory.MONSTER),
		PIG(90, MobCategory.ANIMAL),
		SHEEP(91, MobCategory.ANIMAL),
		COW(92, MobCategory.ANIMAL),
		CHICKEN(93, MobCategory.ANIMAL),
		SQUID(94, MobCategory.ANIMAL),
		WOLF(95, MobCategory.ANIMAL),
		MOOSHROOM(96, MobCategory.ANIMAL),
		OCELOT(98, MobCategory.ANIMAL),
		VILLAGER(120, MobCategory.NPC),
		UNKNOWN(0, MobCategory.UNKNOWN);
		
		MobType(int id, MobCategory category)
		{
			this.id = id;
			this.category = category;
		}
		
		private int id = 0;
		private MobCategory category = MobCategory.UNKNOWN;
		
		public int getEntityId()
		{
			return this.id;
		}
		
		public MobCategory getCategory()
		{
			return this.category;
		}
		
		public String getPermName()
		{
			return this.name().toLowerCase();
		}
		
		public String getProperName()
		{
			return NoSpawnEggs.getLocaleConfig().getString("entity.EID" + this.getEntityId(), NoSpawnEggs.getLocaleConfig().getString("entity.0", "Entity"));
		}
	}
	
	enum MobCategory
	{
		MONSTER, ANIMAL, NPC, UNKNOWN
	}
	
	public static MobType getMobTypeFromId(int id)
	{
		for (MobType mobType : MobType.values())
		{
			if (id == mobType.getEntityId())
				return mobType;
		}
		
		return MobType.UNKNOWN;
	}
}
