package me.goosemonkey.NoSpawnEggsV2;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.event.Listener;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.java.JavaPlugin;

public class NoSpawnEggs extends JavaPlugin
{
	static ConfigMain mainConfig;
	static ConfigLocale localeConfig;
	
	@Override
	public void onEnable()
	{
		if (this.getServer().getPluginManager().getPlugin("NoSpawnEggs") != null)
		{
			Plugin oldPlugin = this.getServer().getPluginManager().getPlugin("NoSpawnEggs");
			
			this.getServer().getLogger().severe("Previous version of NoSpawnEggs plugin found! Disabling 'NoSpawnEggs v" + oldPlugin.getDescription().getVersion() + "'.");
			
			this.getServer().getPluginManager().disablePlugin(oldPlugin);
		}
		
		NoSpawnEggs.mainConfig = new ConfigMain(this);
		NoSpawnEggs.localeConfig = new ConfigLocale(this);
		
		this.registerListenerClass(PluginVersion.LIGHT, new ListenerPlayerSpawnEgg());
		this.registerListenerClass(PluginVersion.LIGHT, new ListenerDispenserSpawnEgg());
		this.registerListenerClass(PluginVersion.MODERATE, new ListenerPlayerGolem());
		this.registerListenerClass(PluginVersion.MODERATE, new ListenerPlayerChickenEgg());
		this.registerListenerClass(PluginVersion.MODERATE, new ListenerDispenserChickenEgg());
		this.registerListenerClass(PluginVersion.FULL, new ListenerPlayerMisc());
		this.registerListenerClass(PluginVersion.FULL, new ListenerDispenserMisc());
	}
	
	/**
	 * Registers a listener after checking that the plugin version allows it
	 * @param required Minimum plugin version this listener needs to work
	 * @param listener The listener
	 */
	void registerListenerClass(PluginVersion required, Listener listener)
	{
		PluginVersion current = NoSpawnEggs.getVersion();
		
		if (required == PluginVersion.FULL && current != PluginVersion.FULL)
			return;
		
		if (required == PluginVersion.MODERATE && current == PluginVersion.LIGHT)
			return;
		
		this.getServer().getPluginManager().registerEvents(listener, this);
	}
	
	/**
	 * Get the plugin's main configuration
	 * @return FileConfiguration for MainConfig.yml
	 */
	public static FileConfiguration getMainConfig()
	{
		return mainConfig.getConfig();
	}

	/**
	 * Get the plugin's locale configuration
	 * @return FileConfiguration for Locale.yml
	 */
	public static FileConfiguration getLocaleConfig()
	{
		return localeConfig.getConfig();
	}
	
	public static boolean hasPermission(CommandSender sender, String perm)
	{
		return sender.hasPermission(perm);
	}
	
	public boolean onCommand(CommandSender sender, Command command, String label, String[] args)
	{
		if (args.length == 0 || args.length >= 2)
		{
			sender.sendMessage(ChatColor.GOLD + String.format(NoSpawnEggs.getLocaleConfig().getString("message.version", "NoSpawnEggs version %s."), NoSpawnEggs.getVersion() + " " + this.getDescription().getVersion()));
			
			
			return true;
		}
		
		if (args[0].equalsIgnoreCase("reload"))
		{
			if (!sender.hasPermission("nse.reload") && !sender.isOp() && !(sender instanceof ConsoleCommandSender))
			{
				sender.sendMessage(ChatColor.GOLD + NoSpawnEggs.getLocaleConfig().getString("message.noReloadPerms", "You don't have permission."));
				
				return true;
			}
			
			NoSpawnEggs.localeConfig.reload();
			NoSpawnEggs.mainConfig.reload();
			
			sender.sendMessage(ChatColor.GOLD + NoSpawnEggs.getLocaleConfig().getString("message.reloaded", "NoSpawnEggs configuration reloaded."));
			
			return true;
		}
		
		return false;
	}
	
	/**
	 * Gets the current version of the plugin, change this to distribute different versions
	 * @return Full, Moderate, or Light
	 */
	public static PluginVersion getVersion()
	{
		return PluginVersion.FULL;
	}
	
	public enum PluginVersion
	{
		FULL, MODERATE, LIGHT
	}
}
